################################################################################
# © Copyright 2022-2023 Zapata Computing Inc.
################################################################################
import numpy as np
import pyLIQTR.sim_methods.quantum_ops as qops
from orquestra.integrations.cirq.conversions import from_openfermion

# At this stage of development we are aware that there are some issues with methods
# 2 and 3 and they do not necessarily yield correct results.
from pyLIQTR.QSP.Hamiltonian import Hamiltonian as pyH

from benchq.conversions import pyliqtr_to_openfermion


def generate_1d_heisenberg_hamiltonian(N):
    # Setting J/h's
    # Adjusting these from zero to nonzero may also increase the circuit depth,
    # since additional terms in the Hamiltonian are introduced.
    J_z = 1.0
    J_x = 1.1 * J_z
    J_y = J_x
    h_x = -1.0 * J_z
    h_y = 0.0 * J_z
    h_z = 0.0 * J_z

    tuples, types, coeffs = qops.params_heisenberg_1d(
        N,
        J_x=J_x,
        J_y=J_y,
        J_z=J_z,
        h_x=h_x,
        h_y=h_y,
        h_z=h_z,
        periodic=False,
    )

    sclf = np.sum(np.abs(coeffs))

    ham_strings = qops.ps_text_full_set(tuples, types, N, Coeffs=coeffs / sclf)
    qsp_H = pyH(ham_strings)
    pauli_sum = from_openfermion(pyliqtr_to_openfermion(qsp_H))
    for term in pauli_sum.terms:
        term.coefficient = term.coefficient.real
    return pauli_sum
